import Foundation
import UIKit
import Yosemite
import WooFoundation

/// The main file for Refund Details data.
/// Must conform to NSObject so it can be the UITableViewDataSource.
///
final class RefundDetailsDataSource: NSObject {
    /// Refund
    ///
    private let refund: Refund

    /// Order
    ///
    private let order: Order

    /// Sections to be rendered
    ///
    private(set) var sections = [Section]()

    /// Refunded order items
    ///
    private var refundedItems: [OrderItemRefund] {
        return refund.items
    }

    /// Currency Formatter
    ///
    private let currencyFormatter = CurrencyFormatter(currencySettings: ServiceLocator.currencySettings)

    /// Products from a Refund
    ///
    var products: [Product] {
        return resultsControllers.products
    }

    /// Refunds initializer
    ///
    init(refund: Refund, order: Order) {
        self.refund = refund
        self.order = order
    }

    /// The results controllers used to display a refund
    ///
    private lazy var resultsControllers: RefundDetailsResultController = {
        return RefundDetailsResultController(siteID: order.siteID)
    }()

    /// Set up results controllers
    ///
    func configureResultsControllers(onReload: @escaping () -> Void) {
        resultsControllers.configureResultsControllers(onReload: onReload)
    }

    /// Refund Amount
    ///
    private var refundAmount: String {
        let formattedRefund = currencyFormatter.formatAmount(refund.amount, with: order.currency)

        return formattedRefund ?? String()
    }

    /// Refund Method
    ///
    private var refundMethod: String {
        guard refund.isAutomated == true else {
            return NSLocalizedString("Refunded manually", comment: "Title of the refund detail cell when the refund was done manually.")
        }

        let refundMethodTemplate = NSLocalizedString("Refunded via %@",
                                                     comment: "It reads, 'Refunded via <payment method>'")
        let refundMethodText = String.localizedStringWithFormat(refundMethodTemplate, order.paymentMethodTitle)

        return refundMethodText
    }

    /// Reason for refund note
    ///
    private var refundReason: String? {
        return refund.reason
    }
}


// MARK: - Conformance to UITableViewDataSource
//
extension RefundDetailsDataSource: UITableViewDataSource {
    func numberOfSections(in tableView: UITableView) -> Int {
        return sections.count
    }

    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return sections[section].rows.count
    }

    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let row = sections[indexPath.section].rows[indexPath.row]
        let cell = tableView.dequeueReusableCell(withIdentifier: row.reuseIdentifier, for: indexPath)
        configure(cell, for: row, at: indexPath)

        return cell
    }
}


// MARK: - Support for UITableViewDelegate
//
extension RefundDetailsDataSource {
    /// Set up table section headings
    ///
    func viewForHeaderInSection(_ section: Int, tableView: UITableView) -> UIView? {
        guard let leftText = sections[section].title else {
            return nil
        }

        let headerID = TwoColumnSectionHeaderView.reuseIdentifier
        guard let headerView = tableView.dequeueReusableHeaderFooterView(withIdentifier: headerID) as? TwoColumnSectionHeaderView else {
            fatalError()
        }

        headerView.leftText = leftText
        headerView.rightText = sections[section].rightTitle

        return headerView
    }
}

// MARK: - Support for UITableViewDataSource
//
private extension RefundDetailsDataSource {
    /// Configure cellForRowAtIndexPath:
    ///
    func configure(_ cell: UITableViewCell, for row: Row, at indexPath: IndexPath) {
        switch cell {
        case let cell as ProductDetailsTableViewCell:
            configureRefundedOrderItem(cell, at: indexPath)
        case let cell as LedgerTableViewCell:
            configureProductsRefund(cell, at: indexPath)
        case let cell as TwoColumnHeadlineFootnoteTableViewCell:
            configureRefundAmount(cell, at: indexPath)
        case let cell as WooBasicTableViewCell:
            configureRefundMethod(cell)
        case let cell as ImageAndTitleAndTextTableViewCell:
            configureRefundNote(cell)
        default:
            fatalError("Unidentified refund details row type")
        }
    }

    /// Setup: Product Cell
    ///
    private func configureRefundedOrderItem(_ cell: ProductDetailsTableViewCell, at indexPath: IndexPath) {
        let refundedItem = refundedItems[indexPath.row]
        let product = lookUpProduct(by: refundedItem.productOrVariationID)
        let itemViewModel = ProductDetailsCellViewModel(refundedItem: refundedItem,
                                                        currency: order.currency,
                                                        product: product)

        let imageService = ServiceLocator.imageService

        cell.selectionStyle = .default
        cell.configure(item: itemViewModel, imageService: imageService)
    }

    /// Setup: ProductsRefund summary Cell
    ///
    private func configureProductsRefund(_ cell: LedgerTableViewCell, at indexPath: IndexPath) {
        cell.selectionStyle = .none
        let viewModel = RefundDetailsViewModel(order: order, refund: refund)
        cell.configure(with: viewModel)
    }

    /// Setup: Refund Amount Cell
    ///
    private func configureRefundAmount(_ cell: TwoColumnHeadlineFootnoteTableViewCell, at indexPath: IndexPath) {
        cell.selectionStyle = .none
        cell.leftText = RowTitle.refundAmount
        cell.rightText = refundAmount
        cell.hideFootnote()
    }

    /// Setup: Refund Method Cell
    ///
    private func configureRefundMethod(_ cell: WooBasicTableViewCell) {
        cell.selectionStyle = .none
        cell.bodyLabel?.text = refundMethod
        cell.applyPlainTextStyle()
    }

    /// Setup: Reason for Refund Note Cell
    ///
    private func configureRefundNote(_ cell: ImageAndTitleAndTextTableViewCell) {
        cell.selectionStyle = .none
        cell.update(with: .imageAndTitleOnly(fontStyle: .body),
                    data: .init(title: refundReason ?? "",
                                textTintColor: .text,
                                imageTintColor: .text,
                                numberOfLinesForTitle: 0,
                                isActionable: false))
    }
}


// MARK: - Lookup products
//
private extension RefundDetailsDataSource {
    func lookUpProduct(by productID: Int64) -> Product? {
        return products.filter({ $0.productID == productID }).first
    }
}


// MARK: - Sections
//
extension RefundDetailsDataSource {
    /// Setup: Sections
    ///
    func reloadSections() {
        let products: Section? = {
            if refundedItems.isEmpty {
                return nil
            }

            var rows: [Row] = Array(repeating: .orderItem, count: refundedItems.count)
            rows.append(.productsRefund)

            return Section(title: SectionTitle.product, rightTitle: SectionTitle.quantity, rows: rows)
        }()

        let details: Section = {
            let rows: [Row] = [.refundAmount, .refundMethod]

            return Section(title: SectionTitle.refundDetails, rightTitle: nil, rows: rows)
        }()

        let reason: Section? = {
            guard refundReason?.isEmpty == false else {
                return nil
            }

            let row: Row = .refundReason

            return Section(title: SectionTitle.refundReason, rightTitle: nil, row: row)
        }()

        sections = [products, details, reason].compactMap { $0 }
    }
}


// MARK: - Constants
//
extension RefundDetailsDataSource {
    /// Section Titles
    ///
    enum SectionTitle {
        static let product = NSLocalizedString("Product", comment: "Product section title")
        static let quantity = NSLocalizedString("Qty", comment: "Quantity abbreviation for section title")
        static let refundDetails = NSLocalizedString("Refund Details", comment: "Refund Details section title")
        static let refundReason = NSLocalizedString("Reason for Refund", comment: "Refund note section title")
    }

    /// Row Titles
    ///
    enum RowTitle {
        static let refundAmount = NSLocalizedString("Refund Amount",
                                                    comment: "Refund Details page > Refund Details section > The label that marks the refunded amount")
    }

    /// Table Rows
    ///
    enum Row: CaseIterable {
        /// Listed in the order they appear on screen
        case orderItem
        case productsRefund
        case refundAmount
        case refundMethod
        case refundReason

        var type: UITableViewCell.Type {
            switch self {
            case .orderItem:
                return ProductDetailsTableViewCell.self
            case .productsRefund:
                return LedgerTableViewCell.self
            case .refundAmount:
                return TwoColumnHeadlineFootnoteTableViewCell.self
            case .refundMethod:
                return WooBasicTableViewCell.self
            case .refundReason:
                return ImageAndTitleAndTextTableViewCell.self
            }
        }

        fileprivate var reuseIdentifier: String {
            type.reuseIdentifier
        }
    }

    /// Table Sections
    ///
    struct Section {
        let title: String?
        let rightTitle: String?
        let footer: String?
        let rows: [Row]

        init(title: String? = nil, rightTitle: String? = nil, footer: String? = nil, rows: [Row]) {
            self.title = title
            self.rightTitle = rightTitle
            self.footer = footer
            self.rows = rows
        }

        init(title: String? = nil, rightTitle: String? = nil, footer: String? = nil, row: Row) {
            self.init(title: title, rightTitle: rightTitle, footer: footer, rows: [row])
        }
    }
}
